{% ckan_extends %}

{%- block meta -%}
    {{ super() }}
    {% if pkg['doi_status'] %}
    <meta name="citation_doi" content="doi:{{ pkg['doi'] }}" />
    <meta name="citation_title" content="{{ pkg['title'] }}" />
    <meta name="citation_author" content="{{ pkg['all_authors'] }}" />
    <meta name="citation_online_date" content="{{ pkg['doi_date_published'] }}" />
    <meta name="citation_publisher" content="{{ pkg['doi_publisher'] }}" />
    {% endif %}
{%- endblock -%}
